#! /usr/bin/python

## phones.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    phones
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

phones
======

"""


import os
from trefnydd.utils import file_IO, stats, location

PLACE = ['_', '-', 'bilabial', 'labio-dental', 'dental-alveolar',
         'alveolar', 'front', 'alveolar-palatal', '',
         'velar', 'back', 'glottal']

MANNER = ['_','-','nasal', 'stop', 'fricative',
          'approximant', 'close_vocalic', '', 'half_close_vocalic',
          'half_open_vocalic', 'open_vocalic']

VOICE = ['_','-','voiceless', '', 'voiced']

ROUNDED = ['_','-','unrounded', '', 'half-rounded', 'rounded']


# the empty string points represent 'undefined' eg 'b' in test frame below


class Phone(object):
    def __init__(self, name='', place=0, manner=0, voice=0, rounded=0):
        # name = eg 'ae', 'hh', 'sh', etc
        self.name = name
        self.place = place
        self.manner = manner
        self.voice = voice
        self.rounded = rounded

    def __repr__(self):
        return "Phone('%s', %s, %s, %s, %s)" \
               % (self.name, self.place, self.manner,
                  self.voice, self.rounded)

    def __str__(self):
        return self.name

    def __ne__(self, other):
        # todo: what about being from different phoneSets!?
        return (self.name != other.name) or \
               (self.place != other.place) or \
               (self.manner != other.manner) or \
               (self.voice != other.voice) or \
               (self.rounded != other.rounded)
        
    def __eq__(self, other):
        # todo: what about being from different phoneSets!?
        return (self.name == other.name) and \
               (self.place == other.place) and \
               (self.manner == other.manner) and \
               (self.voice == other.voice) and \
               (self.rounded == other.rounded)
        

    def __sub__(self, other):
        place_diff_sqd = (self.place - other.place)**2
        manner_diff_sqd = (self.manner - other.manner)**2
        voice_diff_sqd = (self.voice - other.voice)**2
        rounded_diff_sqd = (self.rounded - other.rounded)**2
        # sqrt() would be spuriously accurate, therefore int(sqrt())
        return int(sqrt(place_diff_sqd + manner_diff_sqd
                        + voice_diff_sqd + rounded_diff_sqd))


class PhoneString(list):
    def __init__(self):
        list.__init__(self)
        self.phoneset = None # PHONESET_IPA

    def __repr__(self):
        return 'PhoneString([%s])' \
               % ', '.join([repr(s) for s in self])

    def __str__(self):
        return ' '.join([s.name for s in self])

    def setPhoneset(self, phoneset):
        self.phoneset = phoneset

    def getPhoneset(self):
        return str(self.phoneset)

    def setPhoneString(self, phonestring):
        "arg: string phonestring eg 'ae p l'"
        del self[:]
        for p in phonestring.split():
            self.append(self.phoneset.resolve(p))

    def getPhoneString(self):
        return self.__str__()


class Phoneset(set, location.DataSetContainer):
    def __init__(self, phonelist=[]):
        set.__init__(self)
        location.DataSetContainer.__init__(self)
        self.ext = '.phoneset'
        self.zeroPhone = None
        self.medianPhone = None
        if phonelist:
            self.setPhones(phonelist)
            self.set_zeroPhone()
        self.addPhone(SIL_PHONE)

    def getPreamble(self):
        return """# trefnydd::PhoneSet\n
zeroPhone =   %s
medianPhone = %s
comment =     %s\n\n""" \
        % (self.getZeroPhone().__repr__(),
           self.getMedianPhone().__repr__(),
           self.comment)

    preamble = property(getPreamble)

    def loadNameValuePair(self, name, value):
        if name == 'zeroPhone':
            p = Phone()
            p.loadLine(value)
            self.zeroPhone = p
        elif name == 'medianPhone':
            p = Phone()
            p.loadLine(value)
            self.medianPhone = p
        elif name.lower() == 'comment':
            self.comment = value
        else:
            setattr(self, name, value)

    def loadDataItem(self, line):
        p = eval(line.strip()) #Phone()
        self.add(p)

    def addPhone(self, ph):
        self.add(ph)

    def removePhone(self, ph):
        pass

    def __repr__(self):
        return 'Phoneset([%s])' \
               % ', '.join([repr(s) for s in self])

    def __str__(self):
        return '\n'.join([s.name for s in self])

    def setPhones(self, phonelist):
        for p in phonelist:
            self.add(p)

    def resolve(self, ph_name):
        for p in self:
            if p.name == ph_name:
                return p
        #print 'could not resolve %s' % ph_name
        return self.getMedianPhone()

    def setZeroPhone(self): # todo
        pass
##         if not self.keys():
##             self.zeroPhone = ZERO_PHONE
##         else:
##             like_z = [(p - ZERO_PHONE, p) for p in self]
##             like_z.sort()
##             self.zeroPhone = like_z[0][1]

    def getZeroPhone(self):
        return ZERO_PHONE
        if not self.zeroPhone:
            self.setZeroPhone()
        return self.zeroPhone

    def getMedianPhone(self):
        if not self.medianPhone:
            name = 'MED'
            place = int(stats.median([ph.place for ph in self]))
            manner = int(stats.median([ph.manner for ph in self]))
            voice = int(stats.median([ph.voice for ph in self]))
            rounded = int(stats.median([ph.rounded for ph in self]))
            ph = Phone(name, place, manner, voice, rounded)
            self.addPhone(ph)
            self.medianPhone = ph
        return self.medianPhone

    def getMedianDiff(self):
        vals = [(ph - self.get_zeroPhone()) for ph in self]
        return stats.median(vals)

    def getPhonesByFeatures(self, featureTuple):
        """
        featureTuple = (name, place, manner, voice, rounded)
        poss. have features undefined, etc
        returns list of phones fitting feature pattern
        """
        pass


ZERO_PHONE = Phone('zero', 2, 2, 2 ,2)
SIL_PHONE = Phone('SIL', 0, 0, 0, 0)
# ie SIL_PHONE is effectively Phone(-2, -2, -2, -2)

## PHONESET_IPA = Phoneset()
## PHONESET_IPA.loadPhoneset('ipa.phoneset')




def main():
    pass


if __name__ == '__main__':
    pass


